--******************************************************************************/
--******************************************************************************/
--*                                                                            */
--* Trigger: COPIA_REF_ROD                                                     */
--* Tabela : PRODUTO                                                           */
--*                                                                            */
--* Criao:                                                                   */
--* 17/12/2010 - William Azevedo de Paula                                      */
--*                                                                            */ 
--* Descrio: copia dados da tabela PRODUTO para a tabela REFBIBPROD,         */      
--* e altera a data de ltiama atualizao (DTULTATU) da mesma, para que       */ 
--* ela possa ser utilizada em cache no lugar da tabela PRODUTO;               */
--******************************************************************************/
--******************************************************************************/


IF EXISTS (SELECT * FROM SYSOBJECTS O, SYSUSERS U
           WHERE O.NAME = 'COPIA_REF_PROD' AND
                 TYPE   = 'TR'             AND
	              U.UID  = O.UID       AND
	              U.NAME = USER)
DROP TRIGGER COPIA_REF_PROD
GO

CREATE TRIGGER COPIA_REF_PROD
ON PRODUTO
FOR INSERT, UPDATE
AS
   DECLARE  @V_USUARIO      VARCHAR(40);
   DECLARE  @V_TIPO         VARCHAR(1);
   DECLARE  @V_ITPESQ       VARCHAR(40);
   DECLARE  @V_DESC         VARCHAR(300);
   DECLARE  @V_CDFILLOGOPER VARCHAR(4);
   DECLARE  @V_DTOPER       DATETIME;
/*VARIVEIS DO CURSOR C_CURSOR*/

   DECLARE  @V_ID_COMANDO     VARCHAR(1);

   DECLARE  @V_OLD_CDPRODUTO    VARCHAR(10);
   DECLARE  @V_OLD_DSREFBIBPROD VARCHAR(200);

   DECLARE  @V_NEW_CDPRODUTO    VARCHAR(10);
   DECLARE  @V_NEW_DSREFBIBPROD VARCHAR(200);
   
   DECLARE @V_PRODUTO VARCHAR(10);
 
BEGIN
   
   DECLARE C_CURSOR CURSOR FOR
   SELECT
     'U' IDCOMANDO,
         INSERTED.CDPRODUTO,
         INSERTED.DSREFBIBPROD,
         DELETED.CDPRODUTO,
         DELETED.DSREFBIBPROD
   FROM
      INSERTED,
      DELETED
   WHERE
       INSERTED.CDPRODUTO   = DELETED.CDPRODUTO
   UNION ALL

   SELECT
      'I' IDCOMANDO,
          CDPRODUTO,
          DSREFBIBPROD,
          NULL,
          NULL
   FROM
      INSERTED
   WHERE
      0 = (SELECT COUNT(*) FROM DELETED);

   OPEN C_CURSOR;
   FETCH NEXT FROM C_CURSOR INTO @V_ID_COMANDO,
                                 @V_NEW_CDPRODUTO,
                                 @V_NEW_DSREFBIBPROD,
                                
                                 @V_OLD_CDPRODUTO,
                                 @V_OLD_DSREFBIBPROD

                                
   WHILE (@@FETCH_STATUS = 0) BEGIN

      
         SELECT @V_PRODUTO = CDPRODUTO
         FROM REFBIBPROD
         WHERE CDPRODUTO =  @V_NEW_CDPRODUTO;
      
         IF @V_ID_COMANDO= 'U' /*UPDATING*/ BEGIN
            IF (ISNULL(@V_OLD_DSREFBIBPROD,0)    <> ISNULL(@V_NEW_DSREFBIBPROD,0))BEGIN          

                IF NOT @V_PRODUTO IS NULL BEGIN
                   IF NOT (@V_NEW_DSREFBIBPROD   IS NULL) BEGIN
                      UPDATE REFBIBPROD 
                      SET DSREFBIBPROD = @V_NEW_DSREFBIBPROD,
                          DTULTATU = GETDATE() 
                      WHERE CDPRODUTO = @V_OLD_CDPRODUTO 
                   END
                   ELSE BEGIN
                       DELETE FROM REFBIBPROD WHERE CDPRODUTO = @V_NEW_CDPRODUTO;
                       INSERT INTO DELETECACHE (NMTABLECACHE, DSKEYCACHE, DTULTATUCACH)
			     VALUES ('REFBIBPROD', 'CDPRODUTO=' + @V_NEW_CDPRODUTO+';', GETDATE()); 
                   END;                  
                END
                ELSE BEGIN
                   IF NOT (@V_NEW_DSREFBIBPROD   IS NULL) BEGIN
                      INSERT INTO REFBIBPROD (CDPRODUTO, DSREFBIBPROD, DTULTATU)
                                         VALUES(@V_NEW_CDPRODUTO,@V_NEW_DSREFBIBPROD,GETDATE());  
                   END;
                END;      
            END;
         END
         ELSE BEGIN
            IF @V_ID_COMANDO= 'I' /*UPDATING*/ BEGIN
               IF @V_PRODUTO IS NULL BEGIN
                  IF NOT (@V_NEW_DSREFBIBPROD   IS NULL) BEGIN 
                     INSERT INTO REFBIBPROD (CDPRODUTO, DSREFBIBPROD, DTULTATU)
                                     VALUES(@V_NEW_CDPRODUTO,@V_NEW_DSREFBIBPROD,GETDATE()) 
                  END;
               END
               ELSE BEGIN
                 IF NOT (@V_NEW_DSREFBIBPROD   IS NULL) BEGIN
                    UPDATE REFBIBPROD 
                    SET DSREFBIBPROD = @V_NEW_DSREFBIBPROD,
                                      DTULTATU = GETDATE() 
                    WHERE CDPRODUTO = @V_NEW_CDPRODUTO;
                 END
                 ELSE BEGIN
                    DELETE FROM REFBIBPROD WHERE CDPRODUTO = @V_NEW_CDPRODUTO;
                       INSERT INTO DELETECACHE (NMTABLECACHE, DSKEYCACHE, DTULTATUCACH)
			     VALUES ('REFBIBPROD', 'CDPRODUTO=' + @V_NEW_CDPRODUTO+';', GETDATE());  
                 END;
               END;
            END;
         END;

     
      FETCH NEXT FROM C_CURSOR INTO @V_ID_COMANDO,
        
                                    @V_NEW_CDPRODUTO,
                                    @V_NEW_DSREFBIBPROD,
 
                                    @V_OLD_CDPRODUTO,
                                    @V_OLD_DSREFBIBPROD;

                               


   IF @V_TIPO <> '0' BEGIN
      EXEC NMUSUARIO @V_USUARIO;
      IF NOT @V_DESC IS NULL BEGIN
         IF NOT @V_USUARIO IS NULL BEGIN
            SET @V_DTOPER = GETDATE();
            INSERT INTO LOGOPERA
               (NMTABELA,   DTOPERACAO, NMRESPOPER,
                IDTIPOOPER, CDPESQITEM, DSOPERACAO, CDFILLOGOPER)
            VALUES
               ('PRODUTO',  @V_DTOPER, @V_USUARIO,
                @V_TIPO,      @V_ITPESQ, @V_DESC, @V_CDFILLOGOPER);
         END;
      END;
   END;

   END;
   CLOSE      C_CURSOR;
   DEALLOCATE C_CURSOR;

END;
GO


-- VERSAO --
EXEC ATUVEROBJ 'GERA_PEDIDO_INSP','5.11.000';
GO

